# _*_ coding: utf-8 _*_
#
# HSPyLib v0.10.9
#
# Package: main.hspylib.core.crud.db
"""Package initialization."""

__all__ = [
  'db_repository', 
  'firebase', 
  'mysql', 
  'postgres', 
  'sql_factory'
]



"""
  @package: deployer
   @script: git_utils.py
  @purpose: Provides some git utilities.
  @created: Nov 14, 2019
   @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior
   @mailto: yorevs@hotmail.com
     @site: https://github.com/yorevs/homesetup
  @license: Please refer to <https://opensource.org/licenses/MIT>
"""

from hspylib.modules.cli.vt100.terminal import Terminal


class GitUtils:

    @staticmethod
    def top_level_dir():
        return Terminal.shell_exec('git rev-parse --show-toplevel')

    @staticmethod
    def current_branch():
        return Terminal.shell_exec('git symbolic-ref --short HEAD')

    @staticmethod
    def changelog(from_tag, to_tag):
        return Terminal.shell_exec(f'git log --oneline --pretty=format:%h %ad %s --date=short {from_tag}^..{to_tag}^')

    @staticmethod
    def unreleased():
        latest_tag = Terminal.shell_exec('git describe --tags --abbrev=0 HEAD^')
        return Terminal.shell_exec(f'git log --oneline --pretty=format:%h %ad %s --date=short {latest_tag}..HEAD')

    @staticmethod
    def release_date(tag_name):
        return Terminal.shell_exec(f'git log -1 --format=%ad --date=short {tag_name}')

    @staticmethod
    def tag_list():
        return Terminal.shell_exec('git tag')


if __name__ == '__main__':
    GitUtils.unreleased()



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.10.9
#
# Package: main.hspylib.core
"""Package initialization."""

__all__ = [
  'config', 
  'crud', 
  'enums', 
  'exception', 
  'meta', 
  'model', 
  'tools'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.fetch
      @file: fetch.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Any, Optional

import requests

from hspylib.core.enums.http_method import HttpMethod
from hspylib.core.tools.commons import sysout
from hspylib.modules.fetch.http_response import HttpResponse


def fetch(
        url: str,
        method: HttpMethod = HttpMethod.GET,
        headers: list = None,
        body: Optional[Any] = None,
        silent=True) -> Optional[HttpResponse]:
    """ Do a request specified by method and according to parameters.
    :param url: The url to make the request.
    :param method: The http method to be used [ GET, HEAD, POST, PUT, PATCH, DELETE ].
    :param headers: The http request headers.
    :param body: The http request body (payload).
    :param silent: Omits all informational messages.
    :return:
    """

    url = url if url and url.startswith("http") else 'http://{}'.format(url)
    if not silent:
        sysout('Fetching: method={} table_headers={} body={} url={} ...'.format(
            method, headers if headers else '[]', body if body else '{}', url))
    response = requests.request(url=url, method=method.name, headers=headers, data=body, timeout=3)
    return HttpResponse.of(response)


def head(url: str, headers=None, silent=True) -> Optional[HttpResponse]:
    """Do HEAD request and according to parameters."""
    return fetch(url=url, method=HttpMethod.HEAD, headers=headers, silent=silent)


def get(url: str, headers=None, silent=True) -> Optional[HttpResponse]:
    """Do GET request and according to parameters."""
    return fetch(url=url, headers=headers, silent=silent)


def delete(url: str, headers=None, silent=True) -> Optional[HttpResponse]:
    """Do DELETE request and according to parameters."""
    return fetch(url=url, method=HttpMethod.DELETE, headers=headers, silent=silent)


def post(url: str, body=None, headers=None, silent=True) -> Optional[HttpResponse]:
    """Do POST request and according to parameters."""
    return fetch(url, HttpMethod.POST, headers, body, silent)


def put(url, body=None, headers=None, silent=True) -> Optional[HttpResponse]:
    """Do PUT request and according to parameters."""
    return fetch(url=url, method=HttpMethod.PUT, headers=headers, body=body, silent=silent)


def patch(url, body=None, headers=None, silent=True) -> Optional[HttpResponse]:
    """Do PATCH request and according to parameters."""
    return fetch(url=url, method=HttpMethod.PATCH, headers=headers, body=body, silent=silent)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.core.enums
      @file: exit_code.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.core.enums.enumeration import Enumeration


class ExitCode(Enumeration):
    """ Returned when something went wrong due to any Human interaction """
    ERROR = -1

    """ Returned when something ran successfully without errors """
    SUCCESS = 0

    """ Returned when something that was supposed to work and failed due to unexpected software behaviour """
    FAILED = 1

    # Custom exit codes can be defined here {

    # }

    def __str__(self):
        return f"{self.name}({self.value})"

    def __repr__(self):
        return str(self.value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.core.enum
      @file: enumeration.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from enum import Enum
from typing import Any, List


class Enumeration(Enum):

    @classmethod
    def names(cls) -> List[str]:
        return list(map(lambda e: e.name, cls))

    @classmethod
    def values(cls) -> List[Any]:
        return list(map(lambda e: e.value, cls))

    @classmethod
    def value_of(cls, name: str, ignore_case: bool = True) -> Any:
        if ignore_case:
            found = next(filter(lambda en: en.name.upper() == name.upper(), list(cls)), None)
        else:
            found = next(filter(lambda en: en.name == name, list(cls)), None)
        assert found, f"{name} name is not a valid \"{cls.__name__}\""
        return found

    @classmethod
    def of_value(cls, value: Any, ignore_case: bool = False) -> Any:
        if ignore_case:
            found = next(filter(lambda en: str(en.value).upper() == str(value).upper(), list(cls)), None)
        else:
            found = next(filter(lambda en: en.value == value, list(cls)), None)
        assert found, f"\"{value}\" value does not correspond to a valid \"{cls.__name__}\""
        return found



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.menu.extra.mdashboard
      @file: dashboard_item.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
from typing import Callable

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome


class DashboardItem:
    def __init__(
            self,
            icon: Awesome = None,
            tooltip: str = None,
            on_trigger: Callable = None):
        self.icon = icon
        self.tooltip = tooltip
        self.on_trigger = on_trigger



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.icons.font_awesome
      @file: dashboard_icons.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome


class DashboardIcons(Awesome):
    """
        Dashboard UI icons.
        Codes can be found here:
        - https://fontawesome.com/cheatsheet?from=io
    """

    # @formatter:off
    SEARCH = '\uF002'        # 
    EMAIL = '\uF003'         # 
    LIKE = '\uF004'          # 
    USER = '\uF007'          # 
    MOVIE = '\uF008'         # 
    DASHBOARD = '\uF00A'     # 
    POWER = '\uF011'         # 
    SETTINGS = '\uF013'      # 
    HOME = '\uF015'          # 
    DOCUMENT = '\uF016'      # 
    HISTORY = '\uF017'       # 
    DOWNLOAD = '\uF019'      # 
    ARCHIVE = '\uF01C'       # 
    PLAY = '\uF01D'          # 
    REFRESH = '\uF01E'       # 
    FLAG = '\uF024'          # 
    FAVORITE = '\uF02E'      # 
    PRINTER = '\uF02F'       # 
    CAMERA = '\uF030'        # 
    LIST = '\uF03A'          # 
    PICTURE = '\uF03E'       # 
    EDIT = '\uF040'          # 
    PLUS = '\uF067'          # 
    MINUS = '\uF068'         # 
    ERROR = '\uF06A'         # 
    ALERT = '\uF071'         # 
    AGENDA = '\uF073'        # 
    CHAT = '\uF075'          # 
    SHOPPING = '\uF07A'      # 
    GRAPHS = '\uF080'        # 
    FOLDER = '\uF74A'        # 
    FOLDER_OPEN = '\uF07C'   # 
    EXIT = '\uF08B'          # 
    SEND = '\uF989'          # 黎
    TWITTER = '\uF081'       # 
    FACEBOOK = '\uF082'      # 
    LINKED_IN = '\uF08C'     # 
    GRAPH = '\uF08F'         # 
    SAVE = '\uF0C7'          # 
    CLOUD_DOWN = '\uF0ED'    # 
    CLOUD_UP = '\uF0EE'      # 
    NOTIFICATION = '\uF0F3'  # 
    DATABASE = '\uF1C0'      # 
    CONNECT = '\uF1E6'       # 
    IDENTITY = '\uF2BB'      # 
    DOCKER = '\uF308'        # 
    MUSIC = '\uF3B5'         # 
    SECRET = '\uF49C'        # 
    IE = '\uF7FF'            # 
    GIT = '\uF813'           # 
    # @formatter:on


if __name__ == '__main__':
    DashboardIcons.demo_icons()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.views
      @file: qt_view.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from abc import ABC, abstractmethod

from PyQt5.QtWidgets import QWidget

from hspylib.modules.qt.qt_finder import QtFinder


class QtView(ABC):
    def __init__(self, window: QWidget, parent=None):
        super().__init__()
        self.window = window
        self.parent = parent
        self.qt = QtFinder(self.window)

    @abstractmethod
    def setup_ui(self) -> None:
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.core.enum
      @file: http_code.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from http import HTTPStatus

from hspylib.core.enums.enumeration import Enumeration


class HttpCode(Enumeration):
    # 2xx Success
    OK = HTTPStatus.OK
    CREATED = HTTPStatus.CREATED
    ACCEPTED = HTTPStatus.ACCEPTED
    NON_AUTHORITATIVE_INFORMATION = HTTPStatus.NON_AUTHORITATIVE_INFORMATION
    NO_CONTENT = HTTPStatus.NO_CONTENT
    RESET_CONTENT = HTTPStatus.RESET_CONTENT
    PARTIAL_CONTENT = HTTPStatus.PARTIAL_CONTENT
    MULTI_STATUS = HTTPStatus.MULTI_STATUS
    IM_USED = HTTPStatus.IM_USED
    # 3xx Redirection
    MULTIPLE_CHOICES = HTTPStatus.MULTIPLE_CHOICES
    MOVED_PERMANENTLY = HTTPStatus.MOVED_PERMANENTLY
    FOUND = HTTPStatus.FOUND
    SEE_OTHER = HTTPStatus.SEE_OTHER
    NOT_MODIFIED = HTTPStatus.NOT_MODIFIED
    USE_PROXY = HTTPStatus.USE_PROXY
    TEMPORARY_REDIRECT = HTTPStatus.TEMPORARY_REDIRECT
    PERMANENT_REDIRECT = HTTPStatus.PERMANENT_REDIRECT
    # 4xx Client Error
    BAD_REQUEST = HTTPStatus.BAD_REQUEST
    UNAUTHORIZED = HTTPStatus.UNAUTHORIZED
    PAYMENT_REQUIRED = HTTPStatus.PAYMENT_REQUIRED
    FORBIDDEN = HTTPStatus.FORBIDDEN
    NOT_FOUND = HTTPStatus.NOT_FOUND
    METHOD_NOT_ALLOWED = HTTPStatus.METHOD_NOT_ALLOWED
    NOT_ACCEPTABLE = HTTPStatus.NOT_ACCEPTABLE
    PROXY_AUTHENTICATION_REQUIRED = HTTPStatus.PROXY_AUTHENTICATION_REQUIRED
    REQUEST_TIMEOUT = HTTPStatus.REQUEST_TIMEOUT
    CONFLICT = HTTPStatus.CONFLICT
    GONE = HTTPStatus.GONE
    LENGTH_REQUIRED = HTTPStatus.LENGTH_REQUIRED
    PRECONDITION_FAILED = HTTPStatus.PRECONDITION_FAILED
    PAYLOAD_TOO_LARGE = 413
    REQUEST_URI_TOO_LONG = HTTPStatus.REQUEST_URI_TOO_LONG
    UNSUPPORTED_MEDIA_TYPE = HTTPStatus.UNSUPPORTED_MEDIA_TYPE
    REQUESTED_RANGE_NOT_SATISFIABLE = HTTPStatus.REQUESTED_RANGE_NOT_SATISFIABLE
    EXPECTATION_FAILED = HTTPStatus.EXPECTATION_FAILED
    UNPROCESSABLE_ENTITY = HTTPStatus.UNPROCESSABLE_ENTITY
    LOCKED = HTTPStatus.LOCKED
    FAILED_DEPENDENCY = HTTPStatus.FAILED_DEPENDENCY
    UPGRADE_REQUIRED = HTTPStatus.UPGRADE_REQUIRED
    PRECONDITION_REQUIRED = HTTPStatus.PRECONDITION_REQUIRED
    TOO_MANY_REQUESTS = HTTPStatus.TOO_MANY_REQUESTS
    REQUEST_HEADER_FIELDS_TOO_LARGE = HTTPStatus.REQUEST_HEADER_FIELDS_TOO_LARGE
    CONNECTION_CLOSED_WITHOUT_RESPONSE = 444
    UNAVAILABLE_FOR_LEGAL_REASONS = 451
    CLIENT_CLOSED_REQUEST = 499
    # 5xx Server Error
    INTERNAL_SERVER_ERROR = HTTPStatus.INTERNAL_SERVER_ERROR
    NOT_IMPLEMENTED = HTTPStatus.NOT_IMPLEMENTED
    BAD_GATEWAY = HTTPStatus.BAD_GATEWAY
    SERVICE_UNAVAILABLE = HTTPStatus.SERVICE_UNAVAILABLE
    GATEWAY_TIMEOUT = HTTPStatus.GATEWAY_TIMEOUT
    HTTP_VERSION_NOT_SUPPORTED = HTTPStatus.HTTP_VERSION_NOT_SUPPORTED
    VARIANT_ALSO_NEGOTIATES = HTTPStatus.VARIANT_ALSO_NEGOTIATES
    INSUFFICIENT_STORAGE = HTTPStatus.INSUFFICIENT_STORAGE
    LOOP_DETECTED = HTTPStatus.LOOP_DETECTED
    NOT_EXTENDED = HTTPStatus.NOT_EXTENDED
    NETWORK_AUTHENTICATION_REQUIRED = HTTPStatus.NETWORK_AUTHENTICATION_REQUIRED
    NETWORK_CONNECT_TIMEOUT_ERROR = 599



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.10.9
#
# Package: main.hspylib.modules.cli.icons.font_awesome
"""Package initialization."""

__all__ = [
  'awesome', 
  'dashboard_icons', 
  'form_icons', 
  'widget_icons'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.core.enum
      @file: charset.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.core.enums.enumeration import Enumeration


class Charset(Enumeration):
    ASCII = 'ASCII'
    BIG5 = 'BIG5'
    BIG5HKSCS = 'BIG5HKSCS'
    CP037 = 'CP037'
    CP424 = 'CP424'
    CP437 = 'CP437'
    CP500 = 'CP500'
    CP737 = 'CP737'
    CP775 = 'CP775'
    CP850 = 'CP850'
    CP852 = 'CP852'
    CP855 = 'CP855'
    CP856 = 'CP856'
    CP857 = 'CP857'
    CP860 = 'CP860'
    CP861 = 'CP861'
    CP862 = 'CP862'
    CP863 = 'CP863'
    CP864 = 'CP864'
    CP865 = 'CP865'
    CP866 = 'CP866'
    CP869 = 'CP869'
    CP874 = 'CP874'
    CP875 = 'CP875'
    CP932 = 'CP932'
    CP949 = 'CP949'
    CP950 = 'CP950'
    CP1006 = 'CP1006'
    CP1026 = 'CP1026'
    CP1140 = 'CP1140'
    CP1250 = 'CP1250'
    CP1251 = 'CP1251'
    CP1252 = 'CP1252'
    CP1253 = 'CP1253'
    CP1254 = 'CP1254'
    CP1255 = 'CP1255'
    CP1256 = 'CP1256'
    CP1257 = 'CP1257'
    CP1258 = 'CP1258'
    EUC_JP = 'EUC-JP'
    EUC_JIS_2004 = 'EUC-JIS_2004'
    EUC_JISX0213 = 'EUC-JISX0213'
    EUC_KR = 'EUC-KR'
    GB2312 = 'GB2312'
    GBK = 'GBK'
    GB18030 = 'GB18030'
    HZ = 'HZ'
    ISO2022_JP = 'ISO2022-JP'
    ISO2022_JP_1 = 'ISO2022-JP-1'
    ISO2022_JP_2 = 'ISO2022-JP-2'
    ISO2022_JP_2004 = 'ISO2022-JP-2004'
    ISO2022_JP_3 = 'ISO2022-JP-3'
    ISO2022_JP_EXT = 'ISO2022-JP_EXT'
    ISO2022_KR = 'ISO2022-KR'
    LATIN_1 = 'LATIN-1'
    ISO8859_2 = 'ISO8859_2'
    ISO8859_3 = 'ISO8859-3'
    ISO8859_4 = 'ISO8859-4'
    ISO8859_5 = 'ISO8859-5'
    ISO8859_6 = 'ISO8859-6'
    ISO8859_7 = 'ISO8859-7'
    ISO8859_8 = 'ISO8859-8'
    ISO8859_9 = 'ISO8859-9'
    ISO8859_10 = 'ISO8859-10'
    ISO8859_13 = 'ISO8859-13'
    ISO8859_14 = 'ISO8859-14'
    ISO8859_15 = 'ISO8859-15'
    JOHAB = 'JOHAB'
    KOI8_R = 'KOI8-R'
    KOI8_U = 'KOI8-U'
    MAC_CYRILLIC = 'MAC-CYRILLIC'
    MAC_GREEK = 'MAC-GREEK'
    MAC_ICELAND = 'MAC-ICELAND'
    MAC_LATIN2 = 'MAC-LATIN2'
    MAC_ROMAN = 'MAC-ROMAN'
    MAC_TURKISH = 'MAC-TURKISH'
    PTCP154 = 'PTCP154'
    SHIFT_JIS = 'SHIFT-JIS'
    SHIFT_JIS_2004 = 'SHIFT-JIS_2004'
    SHIFT_JISX0213 = 'SHIFT-JISX0213'
    UTF_16 = 'UTF-16'
    UTF_16_BE = 'UTF-16-BE'
    UTF_16_LE = 'UTF-16-LE'
    UTF_7 = 'UTF-7'
    UTF_8 = 'UTF-8'

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return str(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.promotions
      @file: button_label.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QLabel


class ButtonLabel(QLabel):
    clicked = pyqtSignal(int)

    def __init__(self, parent):
        super().__init__(parent=parent)

    def mousePressEvent(self, ev) -> None:  # pylint: disable=unused-argument
        self.clicked.emit()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.addons.widman
      @file: widman.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import os
import sys
from typing import Any, List

from hspylib.addons.widman.widget import Widget
from hspylib.core.enums.exit_code import ExitCode
from hspylib.core.exception.exceptions import WidgetNotFoundError, WidgetExecutionError
from hspylib.core.meta.singleton import Singleton
from hspylib.core.tools.commons import get_path, syserr
from hspylib.core.tools.text_helper import camelcase
from hspylib.modules.cli.menu.extra.mdashboard.dashboard_item import DashboardItem
from hspylib.modules.cli.menu.extra.mdashboard.mdashboard import mdashboard

HERE = get_path(__file__)


class WidgetManager(metaclass=Singleton):
    """TODO"""

    WIDGETS_PATH = (HERE / "widgets")

    WIDGET_PREFIX = 'widget_'

    class WidgetEntry:
        def __init__(self, file: str, path: str):
            self.module = os.path.splitext(file)[0]
            self.name = camelcase(self.module.replace(WidgetManager.WIDGET_PREFIX, ''))
            self.clazz = f"Widget{self.name.replace('_', '')}"
            self.path = path

        def __str__(self):
            return f"{self.name}: {self.module}.{self.clazz} => {self.path}"

        def __repr__(self):
            return str(self)

    @staticmethod
    def _name_matches(widget_1_name: str, widget_2_name: str):
        return \
            widget_1_name.lower() == widget_2_name.lower() \
            or widget_1_name == widget_2_name.capitalize() \
            or widget_1_name == camelcase(widget_2_name) \
            or widget_1_name.lower() == widget_2_name.lower().replace('_', '')

    def __init__(self, parent: Any):
        self._parent = parent
        self._widgets = []
        self._lookup_paths = os.environ.get('HHS_WIDGETS_PATH', '').split(':')
        self._lookup_paths.insert(0, str(WidgetManager.WIDGETS_PATH))
        list(map(sys.path.append, self._lookup_paths))
        self._load_widgets()

    def execute(self, widget_name: str, widget_args: List[Any]) -> Any:
        """Execute the specified widget"""
        widget = self._find_widget(camelcase(widget_name))
        try:
            exit_code = widget.execute(*widget_args)
            if exit_code == ExitCode.SUCCESS:
                widget.cleanup()
            else:
                raise WidgetExecutionError(f"Widget '{widget_name}' failed to execute. exit_code={exit_code}")
        except Exception as err:
            syserr("Current widget paths: \n{}".format('\n'.join(self._lookup_paths)))
            raise WidgetExecutionError(f"Unable to execute widget '{widget_name}' -> {err}") from err

    def dashboard(self) -> None:
        """Display all available widgets from the widget lookup paths"""
        items = []
        try:
            for widget_entry in self._widgets:
                widget = self._find_widget(widget_entry.name)
                item = DashboardItem(
                    widget.icon(),
                    f"{widget.name()} v{widget.version()}: {widget.tooltip()}",
                    widget.execute
                )
                items.append(item)
            mdashboard(items, 6, 'Please select a widget to execute')
        except Exception as err:
            syserr("Current widget paths: \n{}".format('\n'.join(self._lookup_paths)))
            raise WidgetExecutionError(f"Failed to execute widget :: {err}") from err

    def _load_widgets(self):
        """Search and load all widgets from the widget lookup paths"""
        for path in self._lookup_paths:
            for root, _, files in os.walk(path):
                filtered = list(filter(
                    lambda p: p.startswith(self.WIDGET_PREFIX) and p.endswith('py'), files)
                )
                widgets = list(
                    map(lambda w: self.WidgetEntry(w, f"{root}/{w}"), filtered) # pylint: disable=W0640
                )
                self._widgets.extend(widgets)

    def _find_widget(self, widget_name: str):
        widget_entry = next((w for w in self._widgets if self._name_matches(widget_name, w.name)), None)
        if not widget_entry:
            raise WidgetNotFoundError(
                f"Widget '{widget_name}' was not found on widget lookup paths: {str(self._lookup_paths)}")
        try:
            widget_module = __import__(widget_entry.module)
        except ModuleNotFoundError as err:
            raise WidgetNotFoundError(
                f"Widget '{widget_name}' was not found on widget lookup paths: {str(self._lookup_paths)}") from err
        widget_clazz = getattr(widget_module, widget_entry.clazz)
        widget = widget_clazz()
        assert isinstance(widget, Widget), \
            'All widgets must inherit from "hspylib.addons.widman.widget.Widget"'
        return widget



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.tables
      @file: table_renderer.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import sys
from typing import List, Optional

from hspylib.core.tools.text_helper import TextAlignment, fit_text


class TableRenderer:

    def __init__(
            self,
            table_headers: List[str],
            table_data: Optional[iter],
            table_caption: str = None):
        """
        :param table_headers: table headers to be displayed.
        :param table_data: table record set with the selected rows.
        :param table_caption: table caption to be displayed.
        """
        self.headers = table_headers
        self.rows = table_data if table_data else []
        self.caption = table_caption
        self.header_alignment = TextAlignment.CENTER
        self.cell_alignment = TextAlignment.LEFT
        self.min_column_size = 6
        if self.rows:
            assert len(min(self.rows, key=len)) == len(self.headers), \
                f'Headers and Columns must have the same size: {len(min(self.rows, key=len))} vs {len(self.headers)}'
        self.column_sizes = [
            max(self.min_column_size, len(header)) for header in self.headers
        ]
        self.indexes = range(0, len(self.column_sizes))

    def set_header_alignment(self, alignment: TextAlignment) -> None:
        """
        Set table header justification.
        :param alignment: table header text alignment function.
        :return:
        """
        self.header_alignment = alignment

    def set_cell_alignment(self, alignment: TextAlignment) -> None:
        """
        Set table header justification.
        :param alignment: table cell text alignment function.
        :return:
        """
        self.cell_alignment = alignment

    def set_min_column_size(self, size: int) -> None:
        """
        Set table header justification.
        :param size: minimum table cell size.
        :return:
        """
        self.min_column_size = size

    def adjust_sizes_by_largest_cell(self) -> None:
        """
        Render table based on the maximum size of all cell data.
        :return: None
        """
        for row in self.rows:
            for idx, dummy in enumerate(row):
                self.column_sizes[idx] = max(self.column_sizes[idx], len(str(row[idx])))

    def set_fixed_cell_size(self, width: int) -> None:
        """
        Render table based on a fixed size for all cell data.
        :return: None
        """
        for row in self.rows:
            for idx in range(0, len(row)):
                self.column_sizes[idx] = max(width, self.min_column_size)

    def set_cell_sizes(self, cell_sizes: List[int]) -> None:
        """
        Render table based on a list of fixed sizes.
        :return: None
        """
        assert len(min(self.rows, key=len)) == len(cell_sizes), \
            f'Sizes and Columns must have the same size: {len(min(self.rows, key=len))} vs {len(cell_sizes)}'
        for row in self.rows:
            for idx in range(0, len(row)):
                self.column_sizes[idx] = max(cell_sizes[idx], self.min_column_size)

    def render(self, file=sys.stdout) -> None:
        """
        Render table based on the maximum size of a column header.
        :param file: a file-like object (stream); defaults to the current sys.stdout.
        :return: None
        """
        header_cols = self.__join_header_columns()
        data_cols = self.__join_data_columns()
        table_borders = '+' + ''.join((('-' * (self.column_sizes[idx] + 2) + '+') for idx in self.indexes))
        self.__print_table(table_borders, header_cols, data_cols, file)

    def __join_header_columns(self) -> list:
        cols = [self.header_alignment(self.__header_text(idx), self.column_sizes[idx]) for idx in self.indexes]
        return ['| ' + ' | '.join(cols) + ' |']

    def __join_data_columns(self) -> list:
        return [
            '| ' + ''.join(
                '%s | ' % self.cell_alignment(self.__cell_text(row, idx), self.column_sizes[idx]) for idx
                in self.indexes
            ) for row in self.rows
        ]

    def __header_text(self, idx: int) -> str:
        return fit_text(self.headers[idx], self.__cell_size(idx))

    def __cell_text(self, row: tuple, idx: int) -> str:
        return fit_text(str(row[idx]), self.__cell_size(idx))

    def __cell_size(self, idx: int) -> int:
        return self.column_sizes[idx]

    def __print_table(
            self,
            table_line: str,
            header_cols: List[str],
            data_cols: List[str],
            file=sys.stdout) -> None:
        if self.caption:
            print(table_line, file=file)
            print('| ' + fit_text(self.caption, len(table_line) - 4)
                  .center(len(header_cols[0]) - 4, ' ') + ' |', file=file)
        print(table_line, file=file)
        print('\n'.join(header_cols), file=file)
        print(table_line, file=file)
        print(
            '\n'.join(data_cols) if data_cols
            else '| ' + '<empty>'.center(len(table_line) - 4, ' ') + ' |', file=file
        )
        print(table_line, file=file)



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.10.9
#
# Package: main.hspylib.modules.cli
"""Package initialization."""

__all__ = [
  'application', 
  'icons', 
  'keyboard', 
  'menu', 
  'tables', 
  'vt100'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.menu.extra.mdashboard
      @file: dashboard_item_builder.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Any, Callable, Optional

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome
from hspylib.modules.cli.menu.extra.mdashboard.dashboard_item import DashboardItem


class DashboardItemBuilder:
    def __init__(self, parent: Any):
        self.parent = parent
        self.item = DashboardItem()

    def icon(self, icon: Awesome) -> Any:
        self.item.icon = icon
        return self

    def tooltip(self, tooltip: str) -> Any:
        self.item.tooltip = tooltip
        return self

    def on_trigger(self, on_trigger: Callable) -> Optional[Any]:
        self.item.on_trigger = on_trigger
        return self

    def build(self) -> Any:
        self.parent.items.append(self.item)
        return self.parent



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.10.9
#
# Package: main.hspylib.modules.cli.vt100
"""Package initialization."""

__all__ = [
  'terminal', 
  'vt_100', 
  'vt_codes', 
  'vt_colors', 
  'vt_utils'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.promotions
      @file: entity_table_model.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import logging as log
from typing import Any, Type

from PyQt5.QtCore import QAbstractTableModel, QModelIndex, QVariant, Qt
from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QTableView

from hspylib.core.tools.commons import class_attribute_names, class_attribute_values


class DefaultTableModel(QAbstractTableModel):
    def __init__(self, clazz: Type, headers: list = None, cell_alignments: list = None, table_data: list = None,
                 parent: QTableView = None):
        QAbstractTableModel.__init__(self, parent)
        self.clazz = clazz
        self.table_data = table_data or []
        self.headers = headers or self.headers_by_entity()
        self.cell_alignments = cell_alignments or []
        log.info('{} table_headers={}'.format(clazz.__class__.__name__, '|'.join(self.headers)))

    def data(self, index: QModelIndex, role: int = ...) -> Any:
        entity = class_attribute_values(self.table_data[index.row()].__dict__)[index.column()]
        str_entity = str(entity) if entity else ''
        if role == Qt.DisplayRole:
            return str_entity
        if role == Qt.TextAlignmentRole:
            return self.cell_alignments[index.column()] if self.cell_alignments else Qt.AlignLeft
        if role == Qt.BackgroundColorRole:
            return QVariant() if entity else QColor(230, 230, 230)

        return QVariant()

    def headerData(self, section: int, orientation: Qt.Orientation, role: int = ...) -> Any:
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return self.headers[section].upper() if len(self.headers) >= section else '-'
        if orientation == Qt.Vertical and role == Qt.DisplayRole:
            return section
        return QVariant()

    def headers_by_entity(self) -> tuple:
        return class_attribute_names(self.clazz)

    def rowCount(self, parent: QModelIndex = ...) -> int:  # pylint: disable=unused-argument
        return len(self.table_data) if self.table_data and len(self.table_data) > 0 else 0

    def columnCount(self, parent: QModelIndex = ...) -> int:  # pylint: disable=unused-argument
        return len(self.table_data[0].__dict__.keys()) if self.table_data and len(self.table_data) > 0 else 0

    def row(self, index: QModelIndex) -> Any:
        return self.table_data[index.row()]



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.10.9
#
# Package: main.hspylib.modules.cli.menu
"""Package initialization."""

__all__ = [
  'extra', 
  'factory', 
  'menu', 
  'menu_item', 
  'menu_ui', 
  'menu_utils'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt
      @file: qt_finder.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from abc import ABC
from typing import Optional, Type

from PyQt5.QtCore import QObject
from PyQt5.QtWidgets import QWidget, QStackedWidget, QToolBox, QLabel, QToolButton, QTableView, QLineEdit, QDateEdit, \
    QComboBox, QSpinBox, QDoubleSpinBox, QDialogButtonBox


class QtFinder(ABC):
    @staticmethod
    def find_widget(window: QWidget, widget: Type, name: str) -> Optional[QObject]:
        return window.findChild(widget, name)

    def __init__(self, window: QWidget):
        super().__init__()
        self.window = window

    def find_stacked_widget(self, name: str) -> Optional[QStackedWidget]:
        return QtFinder.find_widget(self.window, QStackedWidget, name)

    def find_tool_box(self, name: str) -> Optional[QToolBox]:
        return QtFinder.find_widget(self.window, QToolBox, name)

    def find_label(self, name: str) -> Optional[QLabel]:
        return QtFinder.find_widget(self.window, QLabel, name)

    def find_tool_button(self, name: str) -> Optional[QToolButton]:
        return QtFinder.find_widget(self.window, QToolButton, name)

    def find_table_view(self, name: str) -> Optional[QTableView]:
        return QtFinder.find_widget(self.window, QTableView, name)

    def find_line_edit(self, name: str) -> Optional[QLineEdit]:
        return QtFinder.find_widget(self.window, QLineEdit, name)

    def find_date_edit(self, name: str) -> Optional[QDateEdit]:
        return QtFinder.find_widget(self.window, QDateEdit, name)

    def find_combo_box(self, name: str) -> Optional[QComboBox]:
        return QtFinder.find_widget(self.window, QComboBox, name)

    def find_spin_box(self, name: str) -> Optional[QSpinBox]:
        return QtFinder.find_widget(self.window, QSpinBox, name)

    def find_double_spin_box(self, name: str) -> Optional[QDoubleSpinBox]:
        return QtFinder.find_widget(self.window, QDoubleSpinBox, name)

    def find_button_box(self, name: str) -> Optional[QDialogButtonBox]:
        return QtFinder.find_widget(self.window, QDialogButtonBox, name)
